#!/usr/bin/perl
###########################################################
# Perl/XML/SVG File Menu Creation Script
# Adam S. Kirschner
# December 27, 2004
#
# Version: 
# This script will read in from an XML file and this Perl script
# will output in SVG a file menu system.
#####################################################################################################
# Modules
#####################################################################################################
use warnings;
use strict;
use XML::Simple;

#####################################################################################################
# Configuration
#####################################################################################################

# path of where the menu xml configuration file is
use constant FILENAME => "menu3.xml";

# background color of the document
use constant BACKGROUND_COLOR => "#000066";

# x and y coordinate of menu
use constant MENU_X => 10;
use constant MENU_Y => 40;

# width and height of menu
use constant MENU_WIDTH => 90;
use constant MENU_HEIGHT => 20;

# color of border around the menu
use constant RECT_STROKE => "#FFFFFF";

# color of menu item
use constant MENU_ITEM_OFF => "#000055";

# color of menu item when hovered
use constant MENU_ITEM_ON => "#0000DD";

# font color 
use constant FONT_COLOR => "#FFFFFF";

# text that would appear before the content if the menu item has a submenu
use constant SUB_MENU_ITEM_PREFIX => "*";

# text that would appear after the content if the menu item has a submenu
use constant SUB_MENU_ITEM_SUFFIX => "";

#####################################################################################################

	# change content type to SVG
	print "Content-type: image/svg+xml\n\n";
	print "<?xml version=\"1.0\" standalone=\"no\"?>\n"; 

	# load the xml file 
	my @xml_data = get_XML_data( FILENAME );
	
	# start printing out the menu
	print_menu( @xml_data );

#########################################################################
# Function: 		get_XML_data
# Purpose:		reads in XML data from File
# Returns:		associative array of XML data.
# Outputs: 		nothing
# Parmeters: 
#
#	filename:	the filename of the menu to be drawn
#########################################################################

sub get_XML_data {
	my ( $filename ) = @_;
	my $xml = new XML::Simple;
	my @data = @{ $xml->XMLin( $filename, ForceArray => 1 )->{"menu"} };
	return @data;
}

#########################################################################
# Function: 		print_menu
# Purpose:		recursive function, prints out a menu
# Returns:		nothing
# Outputs: 		SVG menu code
# Parmeters: 
#
#	xml_data:	the entire XML menu (returned from simplexml_load_file)
#########################################################################
sub print_menu { 

	my ( @xml_data ) = @_;

	my $menu_item_id = -1;
	my $menu_count = @xml_data;

	my ( $MENU_X, $MENU_Y, $MENU_HEIGHT, $MENU_WIDTH, $RECT_STROKE, $MENU_ITEM_OFF, $BACKGROUND_COLOR ) = 
		( MENU_X, MENU_Y, MENU_HEIGHT,  MENU_WIDTH * $menu_count , RECT_STROKE, MENU_ITEM_OFF, BACKGROUND_COLOR );


print <<END_OF_PRINT;
<!DOCTYPE svg PUBLIC "-//W3C//DTD SVG 1.0//EN" "http://www.w3.org/TR/2001/REC-SVG-20010904/DTD/svg10.dtd">
<svg>
	<g id=\"document\">
		<rect x="0" y="0" width="1000" height="1000" fill="$BACKGROUND_COLOR" />
	</g>

	<rect x="$MENU_X" y="$MENU_Y" height="$MENU_HEIGHT" width="$MENU_WIDTH" style="stroke: $RECT_STROKE; fill: $MENU_ITEM_OFF; stroke-width: 1px;" />
END_OF_PRINT

	# go through each menu
	foreach my $menu ( @xml_data ) {

		# set up menu ids
		my @menu_counts;
		$menu_item_id++;

		# x and y coordinate of the next menu
		my $x = MENU_X + $menu_item_id * MENU_WIDTH;
		my $y = MENU_Y;

		# trim the menu's text and the location, may have tabs around it from XML layout.
		my $content = trim( $menu->{"content"} );
		my $href = trim( $menu->{"href"} );

		print "<g id=\"menu_$menu_item_id\">\n";

		# print the main menu item
		print_sub_menu_item( $x, $y, $content, $href, $menu_item_id );

		# start counting
		push( @menu_counts, $menu_count );

		# start printing sub menus
		if( $menu->{"menuitem"} ) {

			# get new menu count and add to counts
			my $new_menu_count = @{ $menu->{"menuitem"} };

			# Add Menu Count To general menu counts. 
			push( @menu_counts, $new_menu_count );

			print_sub_menu( $menu_item_id, \@menu_counts, 0, $x, $y + 1 + MENU_HEIGHT, @{ $menu->{"menuitem"} } );
		}

		print "</g>\n";

	}

	print "</svg>";
}

#########################################################################
# Function: 		print_sub_menu
# Purpose:		recursive function, prints out a menu
# Returns:		nothing
# Outputs: 		SVG menu code
# Parmeters: 
#
#	menu_id: 	id number of what this menu is (counter added to be unique)
#	menu_counts:	the amount of items each generation of menus have
#	recursed:	the amount of times this function was recursed 
#			(used to set actions on grandchildren and children)
#	x:		x-coordinate of menu item
#	y: 		y-coordinate of menu item
#	menu:		the menu itself in an array
##########################################################################
sub print_sub_menu {

	my ( $menu_id, $menu_counts_ref, $recursed, $x, $y, @menu ) = @_;
	
	my @menu_counts = @{ $menu_counts_ref };
	my $menu_counts_count = @menu_counts; # used to go through the amount of menu items there are.

	my $grandparent_menu_count = 3;
	my $parent_menu_count = 2;


	my $new_menu_count = @menu;
	my ( $height, $width, $rect_stroke, $menu_item_off, $counter, $parent_menu, $grandparent_menu ) = ( ( MENU_HEIGHT * $new_menu_count ), MENU_WIDTH, RECT_STROKE, MENU_ITEM_OFF, 0, substr( $menu_id, 0, -1 ), substr( $menu_id, 0, -2 ) );
	
	print "<g id=\"items_$menu_id\" style=\"visibility: hidden;\">\n";
	print "<set attributeName=\"visibility\" to=\"hidden\" begin=\"document.mouseover\" />\n";	

	if ( $recursed == 0 ) {
		for ( my $k = 0; $k < $menu_counts[1]; $k++ ) {
			if ( $k != $menu_id ) {
				print "\t<set attributeName=\"visibility\" to=\"hidden\" begin=\"menu_$k.mouseover\" />\n";
			}
		}
	}
	

	# If I have 2 or more submenus, I have to set actions for all 
	# the my siblings and myself so that if you hover over our grandfather, we will hide
	if ( $recursed > 1 ) {
		for( my $loop = 0; $loop < $menu_counts[$menu_counts_count - $grandparent_menu_count]; $loop++ ) {
			print "<set attributeName=\"visibility\" to=\"hidden\" begin=\"link_$grandparent_menu$loop.mouseover\" />\n";
		}
	}

	# fade in the menu once my parent is hovered on
	print "<animate attributeName=\"fill-opacity\" from=\"0.0\" to=\"1.0\" dur=\"0.5s\" begin=\"link_$menu_id.mouseover\" />\n";
	print "<set attributeName=\"visibility\" to=\"visibile\" begin=\"link_$menu_id.mouseover\" />\n";
	
	# If this has been recursed at all, then there are parents, otherwise there aren't
	if ( $recursed > 0 ) {		

		# I have to set all of my siblings to hide my submenu
		for( my $loop = 0; $loop < $menu_counts[$menu_counts_count - $parent_menu_count]; $loop++ ) {
		
			#make sure to skip this menu, otherwise i'll have mouseover and mouseout events for hidden visibility
			if ( "$parent_menu$loop" ne "$menu_id" ) {
				print " <set attributeName=\"visibility\" to=\"hidden\" begin=\"link_$parent_menu$loop.mouseover\" />\n";
			}

		}
	}	

	# this is the rectangle that goes around all the menu items
	print "<rect x=\"$x\" y=\"$y\" height=\"$height\" width=\"$width\" style=\"stroke: $rect_stroke; fill: $menu_item_off; stroke-width: 1px;\" />\n\n";
	#print Dumper( @menu ); exit; 
	# go through and print out each menu item, each menu item is in an array
	foreach my $menu_item ( @menu ) {

		# the id of this menu, every menu is uniquly identified
		my $menu_item_id = $menu_id . $counter++;

		# this item's content and link
		my $content = trim( $menu_item->{"content"} );
		my $href = trim( $menu_item->{"href"} );
		
		# print the actual menu item
		if ( $menu_item->{"menuitem"} ) { 
			print_sub_menu_item( $x, $y, $content, $href, $menu_item_id, 1 );
		} else {
			print_menu_item( $x, $y, $content, $href, $menu_item_id );
		}
				

		# Detect if another submenu exists
		if( $menu_item->{"menuitem"} ) {

			my $new_sub_menu_count = @{ $menu_item->{"menuitem"} };
			
			# Calculate the location of the submenu
			my $new_x = $x + MENU_WIDTH + 1;
			my $new_y = $y;

			# make an array of all the submenus, it gets used to hide THIS menu, 
			# not the recursed ones (unless if THIS menu is a  grandparent)
			push( @menu_counts, $new_sub_menu_count );
			
			# Recurse, I have more menus to do.
			print_sub_menu( $menu_item_id, \@menu_counts, $recursed + 1,$new_x, $new_y, @{ $menu_item->{"menuitem"} } );

		}

		# get ready to print out next menu (if any)
		$y += MENU_HEIGHT;
	}

	print "</g>\n";


}

#########################################################################
# Function: 		print_menu_item
# Purpose:		prints out a menu item in SVG with default
#			actions such as standard hover affect
# Returns:		nothing
# Outputs: 		SVG menu item code
# Parmeters: 
#	x:		x-coordinate of menu item
#	y: 		y-coordinate of menu item
#	content:	caption, text of menu item
#	href:		link, location when clicked on
#	menu_id: 	id number of this menu item (should be unique)	
#########################################################################

sub print_menu_item { 
	my ( $x, $y, $content, $href, $menu_id ) = @_;
	
	my ( $height, $width, $menu_item_off, $menu_item_on, $font_color ) = ( MENU_HEIGHT, MENU_WIDTH, MENU_ITEM_OFF, MENU_ITEM_ON, FONT_COLOR );

	# Calculate the text location. This is based on a fixed size.
	# (X) 4 and (Y) 14 are adjustment coordinates that may affect 
	# font size. I didn't incorporate text font and size for this reason.
	my $text_x = $x + 4;
	my $text_y = $y + 14;
	

print<<END_OF_PRINT;
<a xlink:href="$href">
	<rect x="$x" y="$y" height="$height" width="$width" fill="$menu_item_off" id="link_$menu_id">
		<set attributeName="fill" to="$menu_item_on" begin="link_$menu_id.mouseover" />
		<set attributeName="fill" to="$menu_item_off" begin="link_$menu_id.mouseout" />
	</rect>
	<text x="$text_x" y="$text_y" fill="$font_color" style="pointer-events: none;">
		$content
	</text>
</a>
END_OF_PRINT
}

#########################################################################
# Function: 		print_sub_menu_item
# Purpose:		prints out a sub menu item in SVG with default
#			actions such as standard hover affect 
#			Note: this does the same as print_sub_menu_item
#			except there are 3 extra set lines that keep
#			this item selected when a child has been selected
# Returns:		nothing
# Outputs: 		SVG sub menu item code
# Parmeters: 
#	x:		x-coordinate of menu item
#	y: 		y-coordinate of menu item
#	content:	caption, text of menu item
#	href:		link, location when clicked on
#	menu_id: 	id number of this menu item (should be unique)	
#	show_text:	this shows/hides the prefix/suffix if one is set
#			(a main menu item does NOT show this extra text)
#########################################################################
sub print_sub_menu_item {
	my ( $x, $y, $content, $href, $menu_id, $show_text ) = @_;
	
	my ( $height, $width, $menu_item_off, $menu_item_on, $font_color ) = ( MENU_HEIGHT, MENU_WIDTH, MENU_ITEM_OFF, MENU_ITEM_ON, FONT_COLOR );

	# check to see whether or not to show the extra text
	if ( $show_text ) {
		$content = SUB_MENU_ITEM_PREFIX . "$content" . SUB_MENU_ITEM_SUFFIX;
	}

	# Calculate the text location. This is based on a fixed size.
	# (X) 4 and (Y) 14 are adjustment coordinates that may affect 
	# font size. I didn't incorporate text font and size for this reason.
	my $text_x = $x + 4;
	my $text_y = $y + 14;
	
print<<END_OF_PRINT;
<a xlink:href="$href">
	<rect x="$x" y="$y" height="$height" width="$width" fill="$menu_item_off" id="link_$menu_id">
		<set attributeName="fill" to="$menu_item_on" begin="link_$menu_id.mouseover" />
		<set attributeName="fill" to="$menu_item_off" begin="link_$menu_id.mouseout" />
		<set attributeName="fill" to="$menu_item_on" begin="items_$menu_id.mouseover" />
		<set attributeName="fill" to="$menu_item_off" begin="items_$menu_id.mouseout" />
		<set attributeName="fill" to="$menu_item_off" begin="document.mouseover" />
	</rect>
	<text x="$text_x" y="$text_y" fill="$font_color" style="pointer-events: none;">
		$content
	</text>
</a>
END_OF_PRINT
}

sub trim {
   my($string)=@_;
   for ($string) {
       s/^\s+//g;
       s/\s+$//g;
   }
   return $string;
}

1;
